<?php

namespace Bydedi\Database;

use Bydedi\Cache\CacheInterface;

interface ConnectionInterface
{
    public function __construct(CacheInterface $cache);

    /**
     * load the table structures from the cache
     *
     * @return void
     */
    public function loadStructures();

    /**
     * get the $structures property
     *
     * @return array
     */
    public function getStructures();

    /**
     * init the connection
     *
     * @return void
     */
    public function connect();

    /**
     * close the connection
     *
     * @return bool
     */
    public function disconnect();

    /**
     * execute a statement SQL
     *
     * @param string $sql    the statement SQL
     * @param int    $type   type of result
     * @param int    $format format of array to return
     *
     * @return bool
     */
    public function query($sql, $type, $format);

    /**
     * Execute a SQL request and return all result
     *
     * @param string $sql the SQL request to execute
     *
     * @return array
     */
    public function getAll($sql);

    /**
     * Execute a SQL request and return the first result
     *
     * @param string $sql the SQL request to execute
     *
     * @return array
     */
    public function getOne($sql);

    /**
     * return the primary key of the table given
     *
     * @param string $table name of the table
     *
     * @return bool
     */
    public function getPrimaryKey($table);

    /**
     * return the struct of the table given
     *
     * @param string $table name of the table
     *
     * @return array
     */
    public function getStructure($table);

    /**
     * Fonction pour sauvegarder les données du tableau $data dans la table $table
     *
     * @param string $table nom de la table dans laquelle il faut insérer des data
     * @param array  $data  tableau de donées a insérer
     *
     * @return mixed
     */
    public function save($table, $data);

    /**
     * @param $value
     * @return string
     */
    public function escape($value);

    /**
     * start transaction SQL
     *
     * @return bool
     */
    public function beginTransaction();

    /**
     * end transaction SQL
     *
     * @return bool
     */
    public function endTransaction();

}