<?php

namespace Bydedi\Routing;

use Bydedi\Http\Exception\RedirectHttpException;
use Psr\Log\LoggerInterface;
use Symfony\Component\Config\Loader\LoaderInterface;
use Symfony\Component\HttpFoundation\Request;
use Symfony\Component\Routing\Exception\ResourceNotFoundException;
use Symfony\Component\Routing\Router as RouterSF;
use Symfony\Component\Routing\RequestContext;

/**
 * Router class
 *
 * PHP Version 5.4
 *
 * @package  Bydedi\Routing
 * @author   Matthieu Bouquet <matthieu@dedi-agency.com>
 *
 */
class RouterI18n extends RouterSF
{

    /**
     * @var \Symfony\Component\Config\Loader\LoaderInterface
     */
    protected $annotationDirectoryLoader;

    /**
     * @param LoaderInterface $loader
     * @param mixed $resource
     * @param array $options
     * @param RequestContext|null $context
     * @param LoggerInterface|null $logger
     */
    public function __construct(
        LoaderInterface $loader,
        $resource,
        array $options = array(),
        RequestContext $context = null,
        LoggerInterface $logger = null
    ) {
        $this->annotationDirectoryLoader = $loader;

        parent::__construct($loader, $resource['controller'], $options, $context, $logger);
    }

    /**
     * @return \Symfony\Component\Config\Loader\LoaderInterface
     */
    public function getAnnotationDirectoryLoader()
    {
        return $this->annotationDirectoryLoader;
    }

    public function generate($name, $parameters = array(), $referenceType = self::ABSOLUTE_PATH)
    {

        if (array_key_exists('_locale', $parameters) == false) {
            $parameters['_locale'] = getenv('APP_LOCALE');
        }

        return parent::generate($name, $parameters, $referenceType);
    }

    /**
     * {@inheritdoc}
     */
    public function matchRequest(Request $request)
    {
        try {
            // try to match the request with url
            $match = parent::matchRequest($request);

            // change the locale in ENV
            if (getenv('APP_LOCALE') != $match['_locale']) {
                putenv("APP_LOCALE=" . $match['_locale']);
            }

            return $match;
        } catch (ResourceNotFoundException $e) {
            // if nothing found, we check if we are on home, and need to redirect to /fr - /en
            if ($request->getPathInfo() == '/') {

                // TODO function to resolve best _locale for the client

                throw new RedirectHttpException($this->generate('home'));
            }

            // otherwise throw the exception to delegate
            throw $e;
        }
    }
}
